﻿using Microsoft.IdentityModel.Clients.ActiveDirectory;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace MyConsoleClient
{
    class Program
    {
        static void Main(string[] args)
        {
            var httpClient = new HttpClient();
            var requestMineType = new MediaTypeWithQualityHeaderValue("application/json");
            httpClient.DefaultRequestHeaders.Accept.Add(requestMineType);

            var clientId = ConfigurationManager.AppSettings["ida:ClientID"];
            var authority = ConfigurationManager.AppSettings["ida:Authority"];
            var resourceId = ConfigurationManager.AppSettings["ida:ResourceId"];
            var endpoint = ConfigurationManager.AppSettings["EndpointAddress"];
            var certName = ConfigurationManager.AppSettings["CertName"];

            var authContext = new AuthenticationContext(authority);

            var clientAssertionCertificate = new ClientAssertionCertificate(clientId, GetX509CertObject(certName));

            //var clientAssertionCertificate = new ClientAssertionCertificate(clientId, LoadX509CertObjectFromFile(@"C:\Temp\MyConsoleClientCert.pfx", "password"));

            var authResult = authContext.AcquireTokenAsync(resourceId, clientAssertionCertificate).Result;

            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", authResult.AccessToken);

            var httpResponse = httpClient.GetAsync(endpoint).Result;

            Console.WriteLine(httpResponse.Content.ReadAsStringAsync().Result);
            Console.ReadLine();
        }

        private static X509Certificate2 GetX509CertObject(string certName)
        {
            X509Store store = new X509Store(StoreLocation.CurrentUser);
            try
            {
                store.Open(OpenFlags.ReadOnly);
                // Place all certificates in an X509Certificate2Collection object.
                var certCollection = store.Certificates;
                // Find unexpired certificates.
                var currentCerts = certCollection.Find(X509FindType.FindByTimeValid, DateTime.Now, false);
                // From the collection of unexpired certificates, find the ones with the correct name.
                var signingCert = currentCerts.Find(X509FindType.FindBySubjectDistinguishedName, certName, false);
                if (signingCert.Count == 0)
                {
                    // No matching certificate found.
                    return null;
                }
                // Return the first certificate in the collection, has the right name and is current.
                return signingCert[0];
            }
            finally
            {
                store.Close();
            }
        }

        private static X509Certificate2 LoadX509CertObjectFromFile(string certFileName, string password)
        {
            var cert = new X509Certificate2();
            cert.Import(certFileName, password, X509KeyStorageFlags.PersistKeySet);
            return cert;
        }
    }
}
